// ============================================================================
/*!
 *	@file		UserApp.cs
 *	@brief		ユーザーアプリケーション
 *	@author		nyamo
 */
// ============================================================================
using System;
using System.Collections.Generic;

using Sce.Pss.Core;
using Sce.Pss.Core.Environment;
using Sce.Pss.Core.Graphics;
using Sce.Pss.Core.Imaging;
using Sce.Pss.Core.Input;
using Sce.Pss.Core.Audio;
using Sce.Pss.HighLevel.UI;

using MsLib;

// ========================================================================
//! AppMainクラス
// ========================================================================
public class UserApp
{
	private static GraphicsContext		graphics;
	private static List<Texture2D>		button_texlist;
	private static List<Texture2D>		number_texlist;

    static Sound						sound;
    static SoundPlayer					soundPlayer;
	static string						soundName = "/Application/resource/sound/he.wav";

	static bool		loop				= true;
	static int		button_id			= 0;
	static int		number_id			= 0;

	// --------------------------------------------------------------------
	//! 初期化
	// --------------------------------------------------------------------
	public static void Init()
	{
		graphics = new GraphicsContext();
		graphics.SetClearColor(0.0f, 0.0f, 0.0f, 0.0f);

		MsDraw.Init(graphics);
		MsDebugPrint.Init();

		button_texlist = new List<Texture2D>();
		number_texlist = new List<Texture2D>();
		
		string[] str_button = {
			"/Application/resource/image/button_off.png",
			"/Application/resource/image/button_on.png",
		};
		
		// ボタン画像
		for (int i = 0; i < str_button.Length; ++i) {
			var image = new Image(str_button[i]);
			image.Decode();

			Texture2D texture = createTexture(image);
			button_texlist.Add(texture);

			int posX = MsDraw.Width - image.Size.Width - 5;
			int posY = MsDraw.Height - image.Size.Height - 5;
			MsDraw.AddSprite("B_" + i.ToString(), new MsSprite(texture, posX, posY));

			image.Dispose();
		}

		// 数字画像
		for (int i = 0; i <= 20; ++i) {
			string path = "/Application/resource/image/number_" + i.ToString("00") + ".png";
			var image = new Image(path);

			image.Decode();

			Texture2D texture = createTexture(image);
			number_texlist.Add(texture);
			MsDraw.AddSprite("N_" + i.ToString(), new MsSprite(texture, 5, 5));

			image.Dispose();
		}
		
		// サウンド
		sound = new Sound(soundName);
		soundPlayer = sound.CreatePlayer();

		//UISystem.Initialize(graphics);
	}

	// --------------------------------------------------------------------
	//! 終了処理
	// --------------------------------------------------------------------
	public static void Term()
	{
        soundPlayer.Stop();
        sound.Dispose();

		foreach (var texture in button_texlist) {
			texture.Dispose();
		}

		foreach (var texture in number_texlist) {
			texture.Dispose();
		}

		MsDebugPrint.Term();
		MsDraw.Term();

		graphics.Dispose();
	}

	// --------------------------------------------------------------------
	//! 実行
	// --------------------------------------------------------------------
	public static void Exec()
	{
		while (loop)
		{
			SystemEvents.CheckEvents();
			graphics.Clear();

			Update();
			Render();

			MsDebugPrint.Draw();
			MsDebugPrint.Clear();

			graphics.SwapBuffers ();
		}
	}

	// --------------------------------------------------------------------
	//! 更新
	// --------------------------------------------------------------------
	public static void Update()
	{
		var gamePadData					= GamePad.GetData(0);
        List<TouchData> touchDataList	= Touch.GetData(0);
		
		// デバッグ情報
		{
			MsDebugPrint.Print("Screen:" +
			                   MsDraw.Width.ToString() +
			                   "x" +
			                   MsDraw.Height.ToString(),
			                   0xFFFF0000, 0, 0);
		}

		// ボタンイベント
		if (gamePadData.ButtonsDown == GamePadButtons.Start) {
			number_id = 0;
		}
		
		// タッチイベント
		foreach (var touch in touchDataList) {

			// ボタンタッチ
			float tx = touch.X + 0.5f;
			float ty = touch.Y + 0.5f;
			float sx = tx * MsDraw.Width;
			float sy = ty * MsDraw.Height;

			if (touch.Status == TouchStatus.Up) {
				button_id = 0;
			}

			if (tx > 0.45f && tx < 0.80f &&
			    ty > 0.30f && ty < 0.64f)
			{
				if (touch.Status == TouchStatus.Down) {
					button_id = 1;

					number_id++;
					if (number_id > 20) number_id = 20;

					soundPlayer.Play();
				}
			}
			else {
				button_id = 0;
			}

			MsDebugPrint.Print("tx=" + tx.ToString(), 0xFFFF0000, 0, 300);
			MsDebugPrint.Print("ty=" + ty.ToString(), 0xFFFF0000, 0, 320);

			MsDebugPrint.Print("sx=" + sx.ToString(), 0xFFFF0000, 0, 360);
			MsDebugPrint.Print("sy=" + sy.ToString(), 0xFFFF0000, 0, 380);
		}

		//UISystem.Update(touchDataList);
	}

	// --------------------------------------------------------------------
	//! 描画
	// --------------------------------------------------------------------
	public static void Render()
	{
		MsDraw.DrawSprite("B_" + button_id.ToString());
		MsDraw.DrawSprite("N_" + number_id.ToString());

		//UISystem.Render();
	}

	// --------------------------------------------------------------------
	//! .
	// --------------------------------------------------------------------
	private static Texture2D createTexture(Image image)
	{
		var texture = new Texture2D(image.Size.Width, image.Size.Height, false, PixelFormat.Rgba);
		texture.SetPixels(0, image.ToBuffer());

		return texture;
	}
}

